
# q-expansion of newform 4998.2.a.s, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4998
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [1], [-1], [0], [5], [1], [1], [6], [6], [6], [-4], [11], [0], [-9], [-4], [-7], [-12], [-6], [13], [4], [13], [15], [-13], [-13], [9], [-4], [1], [0], [-14], [-13], [-2], [6], [-16], [17], [-11], [-4], [14], [22], [17], [14], [18], [0], [21], [16], [8], [-4], [-2], [24], [-4], [7], [-1], [-5], [-22], [13], [-3], [-27], [-18], [-9], [-18], [-18], [13], [-12], [-28], [25], [6], [14], [-15], [34], [-9], [-5], [31], [0], [0], [-4], [-34], [32], [6], [-20], [-30], [26], [-18], [-6], [4], [28], [20], [12], [19], [7], [-10], [-4], [-24], [-25], [-23], [-10], [12], [16], [24], [-20], [20], [23], [2], [-6], [-41], [-26], [-36], [10], [-21], [30], [-16], [-2], [8], [40], [-1], [35], [-20], [3], [-19], [12], [-10], [12], [-6], [-36], [47], [32], [-21], [27], [19], [48], [33], [42], [-12], [18], [-4], [-36], [25], [14], [-32], [-5], [-26], [-26], [-43], [30], [23], [1], [-31], [-25], [26], [0], [-30], [-23], [1], [10], [-52], [-33], [4], [36], [16], [-16], [-52], [-3], [-4], [2], [-4], [-25], [12], [-32], [-9], [40], [52], [3], [34], [19], [4], [-11], [4], [-62], [-38], [-50], [16], [18], [-52], [-42], [-50], [-54], [21], [-4], [39], [14], [-14], [7], [-15], [11], [-15], [-26], [-3], [-28], [-12], [37], [-18], [-50], [-12], [30], [-47], [-38], [3], [-38], [65], [8], [-54], [-14], [52], [70], [-61], [-39], [39], [38], [2], [-67], [34], [69], [-20], [26], [-50], [-17], [13], [28], [-18], [-12], [37], [-16], [36], [-12], [10], [-8], [-74], [-42], [-5], [28], [-68], [-42], [-69], [-28], [-37], [-10], [30], [21], [-64], [28], [-31], [-65], [-30], [8], [72], [-23], [24], [6], [41], [-44], [-73], [32], [-31], [-26], [28], [58], [26], [26], [-30], [55], [28], [27], [-28], [39], [24], [4], [30], [-43], [-64], [-40], [1], [-15], [-66], [1], [43], [-43], [-29], [11], [-38], [28], [-32], [21], [-42], [54], [4], [56], [1], [17], [-81], [40], [-50], [-28], [-56], [-13], [35], [45], [52], [52], [54], [88], [16], [80], [-64], [18], [-33], [76], [-59], [-16], [-84], [-18], [26], [16], [-27], [-10], [48], [90], [-38], [-36], [26], [79], [-46], [54], [-23], [-78], [-74], [63], [-74], [66], [-40], [20], [17], [7], [10], [60], [33], [-50], [27], [15], [-10], [-11], [11], [-73], [34], [-38], [55], [40], [2], [9], [40], [92], [42], [43], [-83], [52], [-69], [-57], [24], [19], [-21], [-48], [2], [48], [64], [-42], [-27], [-32], [-62], [0], [58], [42], [-9], [-56], [-75], [12], [3], [7], [91], [95], [-82], [42], [44], [6], [16], [41], [-20], [-90], [-63], [-86], [29], [102], [18], [-35], [50], [12], [-36], [-52], [100], [-38], [-76], [14], [12], [-12], [-51], [-69], [-78], [54], [-58], [-75], [62], [-98], [26], [-60], [-55], [80], [12], [25], [55], [6], [77], [46], [56], [8]]
